@@ -16,6 +16,7 @@ import errno
 import getopt
 import hashlib
 import iniparse
+import io
 import os
 import re
 import shutil
@@ -24,14 +25,31 @@ import tempfile
 
 if sys.version_info[0] >= 3:
     import shlex as pipes
-    from io import StringIO
     import configparser
 else:
+    import codecs
     import pipes
-    from cStringIO import StringIO
     import ConfigParser as configparser
 
 
+user_encoding = 'utf-8'  # user specified items
+file_encoding = 'utf-8'  # encoding of ini file contents
+
+
+# Python 2/3 wrapper to convert strings to unicode
+try:  # Python 2
+    unicode
+
+    def s2u(s, e=user_encoding):
+        return unicode(s, e)
+    # Also add conversion wrapper for print()
+    sys.stdout = codecs.getwriter(user_encoding)(sys.stdout)
+except NameError:  # Python 3
+    def s2u(s, e=user_encoding):
+        return str(s)
+    unicode = str
+
+
 def error(message=None):
     if message:
         sys.stderr.write(message + '\n')
@@ -69,13 +87,31 @@ class CrudiniInputFilter():
         self.fp = fp
         self.iniopt = iniopt
         self.crudini_no_arg = False
+        self.indented = False
+        self.last_section = 'DEFAULT'
+        self.section_indents = {}
         self.windows_eol = None
+        self.bom = None
         # Note [ \t] used rather than \s to avoid adjusting \r\n when no value
         # Unicode spacing around the delimiter would be very unusual anyway
         self.delimiter_spacing = re.compile(r'(.*?)[ \t]*([:=])[ \t]*(.*)')
+        self.leading_whitespace = re.compile(r'([ \t]+)(.+)')
+        self.replace_leading = re.compile(r'^(.+) ;crudini_indent>(.*)<$',
+                                          flags=re.MULTILINE)
+        self.section_match = re.compile(r'[ \t]*\[([^]]+)\].*')
 
     def readline(self):
         line = self.fp.readline()
+
+        # Strip BOM. iniparse tracks it but simpler for us to replace later
+        # as we're munging the data in various ways.
+        if self.bom is None:
+            if line and line[0] == u'\ufeff':
+                line = line[1:]
+                self.bom = True
+            else:
+                self.bom = False
+
         # XXX: This doesn't handle ;inline comments.
         # Really should be done within iniparse.
 
@@ -87,13 +123,26 @@ class CrudiniInputFilter():
             else:
                 self.windows_eol = os.name == 'nt'
 
-        if line and line[0] not in '[ \t#;\n\r':
+        if line.strip() and line[0] not in '#;%':
 
-            if '=' not in line and ':' not in line:
+            if 'ignoreindent' in self.iniopt:
+                section = line.lstrip()[0] == '['
+            else:
+                section = line[0] == '['
+            if section:
+                section_name = self.section_match.sub(r'\1', line.rstrip())
+                if not section_name:
+                    return line
+                self.last_section = section_name
+
+            if line[0] in ' \t' and 'ignoreindent' not in self.iniopt:
+                return line
+
+            if not section and '=' not in line and ':' not in line:
                 self.crudini_no_arg = True
                 line = line.rstrip() + ' = crudini_no_arg\n'
 
-            if 'nospace' in self.iniopt:
+            if not section and 'nospace' in self.iniopt:
                 # Convert _all_ existing params. New params are handled in
                 # the iniparse specialization in CrudiniConfigParser()
 
@@ -103,6 +152,21 @@ class CrudiniInputFilter():
                 # But if need to remove the spacing, then should for all params
 
                 line = self.delimiter_spacing.sub(r'\1\2\3', line)
+            elif not section and 'space' in self.iniopt:
+                # Convert _all_ existing params. New params will be correct
+
+                line = self.delimiter_spacing.sub(r'\1 \2 \3', line)
+
+            if line[0] in ' \t':
+                self.indented = True
+
+                # Set default indent for section to last indent
+                leading_ws = self.leading_whitespace.sub(r'\1', line.rstrip())
+                self.section_indents[self.last_section] = leading_ws
+
+                # match leading spaces and put in trailing ;crudini_indent=...
+                reorder_ws = r'\2 ;crudini_indent>\1<'
+                line = self.leading_whitespace.sub(reorder_ws, line)
 
         return line
 
@@ -117,7 +181,7 @@ class AddDefaultSection(CrudiniInputFilter):
     def readline(self):
         if self.first:
             self.first = False
-            return '[%s]' % iniparse.DEFAULTSECT
+            return s2u('[%s]' % iniparse.DEFAULTSECT)
         else:
             return CrudiniInputFilter.readline(self)
 
@@ -177,14 +241,12 @@ class LockedFile(FileLock):
 
         self.fp_cmp = None
         self.filename = filename
-        self.operation = operation
 
         FileLock.__init__(self, operation != "--get")
 
         atexit.register(self.delete)
 
         open_mode = os.O_RDONLY
-        open_args = {}
         if operation != "--get":
             # We're only reading here, but we check now for write
             # permissions we'll need in --inplace case to avoid
@@ -195,13 +257,11 @@ class LockedFile(FileLock):
             if create and operation != '--del':
                 open_mode += os.O_CREAT
 
-            # Don't convert line endings, so we maintain CRLF in files
-            if sys.version_info[0] >= 3:
-                open_args = {'newline': ''}
-
         try:
-            self.fp = os.fdopen(os.open(self.filename, open_mode, 0o666),
-                                **open_args)
+            # Note we open in binary mode to avoid newline processing,
+            # and also to give more control over the decoding process later.
+            # This avoids platform encoding inconsistencies as per PEP 597.
+            self.fp = os.fdopen(os.open(self.filename, open_mode, 0o666), 'rb')
             if inplace:
                 # In general readers (--get) are protected by file_replace(),
                 # but using read lock here gives AC of the ACID properties
@@ -213,7 +273,7 @@ class LockedFile(FileLock):
                 while True:
                     self.lock()
                     fpnew = os.fdopen(os.open(self.filename, open_mode, 0o666),
-                                      **open_args)
+                                      'rb')
                     if (os.name == 'nt' or
                        os.path.sameopenfile(self.fp.fileno(), fpnew.fileno())):
                         # Note we don't fpnew.close() here as that would break
@@ -232,7 +292,7 @@ class LockedFile(FileLock):
             # We don't exit early here so that --verbose is also handled.
             if create and operation == '--del' \
                and e.errno in (errno.ENOTDIR, errno.ENOENT):
-                self.fp = StringIO('')
+                self.fp = io.BytesIO(b'')
             else:
                 error(str(e))
                 sys.exit(1)
@@ -259,7 +319,7 @@ class CrudiniConfigParser(iniparse.RawConfigParser):
         # We ignore lines starting with '%' which mercurial uses to include
         iniparse.change_comment_syntax('%;#', allow_rem=False)
         if preserve_case:
-            self.optionxform = str
+            self.optionxform = lambda x: x
         # Adjust iniparse separator to default to no space around equals
         # Note this does NOT convert existing params with spaces,
         # that's done in CrudiniInputFilter.readline().
@@ -362,12 +422,16 @@ class PrintSh(Print):
         return True
 
     def name_value(self, name, value, section=None):
-        if not PrintSh._valid_sh_identifier(name):
-            error('Invalid sh identifier: %s' % name)
+        if section and name:
+            identifier = "%s_%s" % (section, name)
+        else:
+            identifier = name
+        if not PrintSh._valid_sh_identifier(identifier):
+            error('Invalid sh identifier "%s"' % identifier)
             sys.exit(1)
         if value == 'crudini_no_arg':
             value = ''
-        sys.stdout.write("%s=%s\n" % (name, pipes.quote(value)))
+        sys.stdout.write("%s=%s\n" % (identifier, pipes.quote(value)))
 
 
 class Crudini():
@@ -375,10 +439,13 @@ class Crudini():
         param = value = vlist = listsep = verbose = None
 
     locked_file = None
-    section_explicit_default = False
+    section_explicit_default = None
     data = None
     conf = None
     added_default_section = False
+    default_adjust = False
+    removed_section = False
+    ini_section_blanks = []
     _print = None
 
     # The following exits cleanly on Ctrl-C,
@@ -430,10 +497,8 @@ class Crudini():
                 st = os.stat(name)
                 os.fchown(f, st.st_uid, st.st_gid)
 
-            if sys.version_info[0] >= 3:
-                os.write(f, bytearray(data, 'utf-8'))
-            else:
-                os.write(f, data)
+            os.write(f, bytearray(data, file_encoding))
+
             # We assume the existing file is persisted,
             # so sync here to ensure new data is persisted
             # before referencing it.  Otherwise the metadata could
@@ -477,13 +542,9 @@ class Crudini():
          - Less Durable as existing data truncated before I/O completes.
          - Requires write access to file rather than write access to dir.
         """
-        # Don't convert line endings, so we maintain CRLF in files
-        open_args = {}
-        if sys.version_info[0] >= 3:
-            open_args = {'newline': ''}
 
-        with open(name, 'w', **open_args) as f:
-            f.write(data)
+        with open(name, 'wb') as f:
+            f.write(bytearray(data, file_encoding))
             f.flush()
             os.fsync(f.fileno())
 
@@ -556,6 +617,8 @@ SECTION can be empty ("") or "DEFAULT" in which case,
 params not in a section, i.e. global parameters are operated on.
 If 'DEFAULT' is used with --set, an explicit [DEFAULT] section is added.
 
+Multiple --set|--del|--get operations for a config_file can be specified.
+
 Options:
 
   --existing[=WHAT]  For --set, --del and --merge, fail if item is missing,
@@ -565,6 +628,9 @@ Options:
                        Formats are 'sh','ini','lines'
   --ini-options=OPT  Set options for handling ini files.  Options are:
                        'nospace': use format name=value not name = value
+                       'space': ensure name = value format
+                       'sectionspace': ensure one blank line between sections
+                       'ignoreindent': ignore leading whitespace
   --inplace          Lock and write files in place.
                        This is not atomic but has less restrictions
                        than the default replacement method.
@@ -579,6 +645,21 @@ Options:
         )
         sys.exit(exitval)
 
+    def set_operation(self, operation):
+        self.mode = None
+        self.cfgfile = self.section = self.param = self.value = None
+        try:
+            self.mode = operation[0]
+            self.cfgfile = operation[1]
+            # Convert the following to unicode as
+            # we process in unicode explicitly in python2.
+            # Not needed on python3 where all strings are unicode.
+            self.section = s2u(operation[2])
+            self.param = s2u(operation[3])
+            self.value = s2u(operation[4])
+        except IndexError:
+            pass
+
     def parse_options(self):
 
         # Handle optional arg to long option
@@ -589,41 +670,64 @@ Options:
             elif opt == '--':
                 break
 
+        long_options = [
+            'del',
+            'existing=',
+            'format=',
+            'get',
+            'help',
+            'ini-options=',
+            'inplace',
+            'list',
+            'list-sep=',
+            'merge',
+            'output=',
+            'set',
+            'verbose',
+            'version'
+        ]
+
+        # Group args into options and operations
+        options = []
+        operations = []
+        next_is_option_param = False
+        for i, opt in enumerate(sys.argv[1:]):
+            if next_is_option_param:
+                options.append(opt)
+                next_is_option_param = False
+            elif opt in ('--get', '--set', '--del', '--merge'):
+                operations.append([opt])
+            elif opt == '--':
+                if operations:
+                    operations[-1].extend(sys.argv[i+2:])
+                # else discard as was done before multi operation support
+                break
+            elif opt.startswith('--'):
+                options.append(opt)
+                if '=' not in opt and opt[2:]+'=' in long_options:
+                    next_is_option_param = True
+            else:
+                if operations:
+                    operations[-1].append(opt)
+                else:
+                    error('Unknown operation: %s' % opt)
+                    break
+
         try:
-            long_options = [
-                'del',
-                'existing=',
-                'format=',
-                'get',
-                'help',
-                'ini-options=',
-                'inplace',
-                'list',
-                'list-sep=',
-                'merge',
-                'output=',
-                'set',
-                'verbose',
-                'version'
-            ]
-            opts, args = getopt.getopt(sys.argv[1:], '', long_options)
+            opts, args = getopt.getopt(options, '', long_options)
         except getopt.GetoptError as e:
             error(str(e))
             self.usage(1)
 
+        self.iniopt = ()
         for o, a in opts:
             if o in ('--help',):
                 self.usage(0)
             elif o in ('--version',):
-                print('crudini 0.9.4')
+                print('crudini 0.9.6')
                 sys.exit(0)
             elif o in ('--verbose',):
                 self.verbose = True
-            elif o in ('--set', '--del', '--get', '--merge'):
-                if self.mode:
-                    error('One of --set|--del|--get|--merge can be specified')
-                    self.usage(1)
-                self.mode = o
             elif o in ('--format',):
                 self.fmt = a
                 if self.fmt not in ('sh', 'ini', 'lines'):
@@ -632,9 +736,13 @@ Options:
             elif o in ('--ini-options',):
                 self.iniopt = a.split(',')
                 for opt in self.iniopt:
-                    if opt not in ('nospace'):
+                    if opt not in ('', 'nospace', 'space', 'sectionspace',
+                                   'ignoreindent'):
                         error('--ini-options not recognized: %s' % opt)
                         self.usage(1)
+                if 'nospace' in self.iniopt and 'space' in self.iniopt:
+                    error('--ini-options=space,nospace are mutually exclusive')
+                    sys.exit(1)
             elif o in ('--existing',):
                 self.update = a or 'param'  # 'param' implies all must exist
                 if self.update not in ('file', 'section', 'param'):
@@ -649,63 +757,10 @@ Options:
             elif o in ('--output',):
                 self.output = a
 
-        if not self.mode:
+        if not operations:
             error('One of --set|--del|--get|--merge must be specified')
             self.usage(1)
 
-        try:
-            self.cfgfile = args[0]
-            self.section = args[1]
-            self.param = args[2]
-            self.value = args[3]
-        except IndexError:
-            pass
-
-        if not self.output:
-            self.output = self.cfgfile
-
-        if file_is_closed(sys.stdout) \
-           and (self.output == '-' or self.mode == '--get'):
-            error("stdout is closed")
-            sys.exit(1)
-
-        if self.cfgfile is None:
-            self.usage(1)
-        if self.section is None and self.mode in ('--del', '--set'):
-            self.usage(1)
-        if self.param is not None and self.mode in ('--merge',):
-            self.usage(1)
-        if self.value is not None and self.mode not in ('--set',):
-            if not (self.mode == '--del' and self.vlist):
-                error('A value should not be specified with %s' % self.mode)
-                self.usage(1)
-
-        if self.mode == '--merge' and self.fmt == 'sh':
-            # I'm not sure how useful is is to support this.
-            # printenv will already generate a mostly compat ini format.
-            # If you want to also include non exported vars (from `set`),
-            # then there is a format change.
-            error('sh format input is not supported at present')
-            sys.exit(1)
-
-        # Protect against generating non parseable ini files
-        if self.section and ('[' in self.section or ']' in self.section):
-            error("section names should not contain '[' or ']': %s" %
-                  self.section)
-            sys.exit(1)
-        if self.param and self.param.startswith('['):
-            error("param names should not start with '[': %s" % self.param)
-            sys.exit(1)
-
-        if not self.iniopt:
-            self.iniopt = ()
-        # A "param=with=equals = value" line can not be found with --get
-        # so avoid the ambiguity.  Restrict to 'nospace' to allow hack in
-        # https://github.com/pixelb/crudini/issues/33#issuecomment-1151253988
-        if 'nospace' in self.iniopt and self.param and '=' in self.param:
-            error("param names should not contain '=': %s" % self.param)
-            sys.exit(1)
-
         if self.fmt == 'lines':
             self._print = PrintLines()
         elif self.fmt == 'sh':
@@ -718,8 +773,96 @@ Options:
         else:
             self._print = Print()
 
+        # Validate all operations combinations
+        for o in operations:
+            if self.mode and self.mode != o[0]:
+                mixable = ('--set', '--del')
+                if self.mode not in mixable or o[0] not in mixable:
+                    error("--get|--merge modes can't be mixed"
+                          "with --set|--del")
+                    self.usage(1)
+            elif self.mode == '--merge':
+                error("--merge mode can't be repeated")
+                self.usage(1)
+
+            if self.cfgfile and len(o) > 1 and self.cfgfile != o[1]:
+                error("Can't operate on multiple files")
+                self.usage(1)
+
+            self.set_operation(o)
+
+            if self.cfgfile is None:
+                self.usage(1)
+            if self.section is None and self.mode in ('--del', '--set'):
+                self.usage(1)
+            if self.param is not None and self.mode in ('--merge',):
+                self.usage(1)
+            if self.value is not None and self.mode not in ('--set',):
+                if not (self.mode == '--del' and self.vlist):
+                    error('A value should not be specified with %s'
+                          % self.mode)
+                    self.usage(1)
+
+            # Convert secion '' to 'DEFAULT',
+            # ensuring no conflicting DEFAULT section specs
+            if self.section_explicit_default is None:
+                if self.section == '':
+                    o[2] = self.section = iniparse.DEFAULTSECT
+                    self.section_explicit_default = False
+                elif self.section == iniparse.DEFAULTSECT:
+                    self.section_explicit_default = True
+            elif self.section is not None:
+                if ((self.section == '' and self.section_explicit_default)
+                    or (self.section == iniparse.DEFAULTSECT
+                        and not self.section_explicit_default)):
+                    error("Conflicting %s section specifications" %
+                          iniparse.DEFAULTSECT)
+                    sys.exit(1)
+                elif self.section == '':
+                    o[2] = self.section = iniparse.DEFAULTSECT
+
+            if self.mode == '--merge' and self.fmt == 'sh':
+                # I'm not sure how useful is is to support this.
+                # printenv will already generate a mostly compat ini format.
+                # If you want to also include non exported vars (from `set`),
+                # then there is a format change.
+                error('sh format input is not supported at present')
+                sys.exit(1)
+
+            # Protect against generating non parseable ini files
+            if self.section and ('[' in self.section or ']' in self.section):
+                error("section names should not contain '[' or ']': %s" %
+                      self.section)
+                sys.exit(1)
+            if self.param and self.param.startswith('['):
+                error("param names should not start with '[': %s" % self.param)
+                sys.exit(1)
+
+            # A "param=with=equals = value" line can not be found with --get
+            # so avoid the ambiguity.  Note this precludes the "nospace" hack
+            # in https://github.com/pixelb/crudini/issues/33#issuecomment-\
+            # 1151253988
+            if self.param and '=' in self.param:
+                error("param names should not contain '=': %s" % self.param)
+                if 'nospace' not in self.iniopt:
+                    error("Use --ini-options=nospace if you want that format")
+                sys.exit(1)
+
+        if self.section_explicit_default is None:
+            self.section_explicit_default = False
+
+        if not self.output:
+            self.output = self.cfgfile
+
+        if file_is_closed(sys.stdout) \
+           and (self.output == '-' or self.mode == '--get'):
+            error("stdout is closed")
+            sys.exit(1)
+
+        return operations
+
     def _has_default_section(self):
-        fp = StringIO(self.data)
+        fp = io.StringIO(self.data)
         for line in fp:
             if line.startswith('[%s]' % iniparse.DEFAULTSECT):
                 return True
@@ -727,10 +870,7 @@ Options:
 
     def _chksum(self, data):
         h = hashlib.sha256()
-        if sys.version_info[0] >= 3:
-            h.update(bytearray(data, 'utf-8'))
-        else:
-            h.update(data)
+        h.update(bytearray(data, file_encoding))
         return h.digest()
 
     def _parse_file(self, filename, add_default=False, preserve_case=False):
@@ -740,9 +880,10 @@ Options:
                 # Doing it here will avoid rereads on reparse and support
                 # correct parsing of stdin
                 if filename == '-':
-                    self.data = sys.stdin.read()
+                    ifp = os.fdopen(sys.stdin.fileno(), 'rb')
                 else:
-                    self.data = self.locked_file.fp.read()
+                    ifp = self.locked_file.fp
+                self.data = ifp.read().decode(file_encoding)
                 if self.mode != '--get':
                     # compare checksums to flag any changes
                     # (even spacing or case adjustments) with --verbose,
@@ -754,7 +895,8 @@ Options:
                 else:
                     self.newline_at_start = False
 
-            fp = StringIO(self.data)
+            # newline='' =-> don't convert line endings
+            fp = io.StringIO(self.data, newline='')
             if add_default:
                 fp = AddDefaultSection(fp, self.iniopt)
             else:
@@ -765,7 +907,11 @@ Options:
                                          'nospace' not in self.iniopt))
             conf.readfp(fp)
             self.crudini_no_arg = fp.crudini_no_arg
+            self.indented = fp.indented
+            self.replace_leading = fp.replace_leading
+            self.section_indents = fp.section_indents
             self.windows_eol = fp.windows_eol
+            self.bom = fp.bom
             return conf
         except EnvironmentError as e:
             error(str(e))
@@ -814,6 +960,22 @@ Options:
 
     def set_name_value(self, section, param, value):
         curr_val = None
+        ignore_indent = 'ignoreindent' in self.iniopt
+
+        # Since indents stripped on read, ensure no ambiguities
+        # Also allow to set default indent on params with explicit indent
+        # We don't support this for sections as in all cases they
+        # can have [  leading spaces] in their names. TODO: Perhaps should
+        # support specifying '  [spaces before brackets]' for sections?
+        if ignore_indent and param:
+            stripped_param = param.lstrip()
+            current_indent = self.section_indents.get(section or 'DEFAULT')
+            if not current_indent:
+                leading_ws = param[:len(param)-len(stripped_param)]
+                if leading_ws:
+                    self.indented = True
+                    self.section_indents[section] = leading_ws
+            param = stripped_param
 
         if self.update in ('param', 'section'):
             if param is None:
@@ -836,11 +998,21 @@ Options:
             if self.mode == "--del":
                 return
             else:
+                # Adjust to allow adding a "default" section (issue #80)
+                skip_section_add = False
+                if section.lower() == "default":
+                    section = "crudini_default_adjust_%s" % section
+                    self.default_adjust = True
+                    if self.conf.has_section(section):  # We already added
+                        skip_section_add = True
+
                 # Note this always adds a '\n' before the section name
                 # resulting in double spaced sections or blank line at
                 # the start of a new file to which a new section is added.
-                # We handle the empty file case at least when writing.
-                self.conf.add_section(section)
+                # List the sections here to adjust when writing.
+                if not skip_section_add:
+                    self.ini_section_blanks.append(section)
+                    self.conf.add_section(section)
 
         if param is not None:
             try:
@@ -867,6 +1039,11 @@ Options:
                     # Otherwise use a delimeter
                     value = ''
 
+            # Add a default indent through an inline comment, to later replace
+            section_indent = self.section_indents.get(section)
+            if curr_val is None and ignore_indent and section_indent:
+                value += ' ;crudini_indent>%s<' % section_indent
+
             if self.vlist:
                 value = self.update_list(
                     curr_val,
@@ -917,9 +1094,11 @@ Options:
                 for name in self.conf.defaults():
                     self.conf.remove_option(iniparse.DEFAULTSECT, name)
             else:
-                if not self.conf.remove_section(self.section) \
-                   and self.update in ('param', 'section'):
-                    raise configparser.NoSectionError(self.section)
+                if not self.conf.remove_section(self.section):
+                    if self.update in ('param', 'section'):
+                        raise configparser.NoSectionError(self.section)
+                else:
+                    self.removed_section = True
         elif self.value is None:
             try:
                 if not self.conf.remove_option(self.section, self.param) \
@@ -934,7 +1113,7 @@ Options:
     def command_get(self):
         """Output a section/parameter"""
 
-        if self.fmt != 'lines':
+        if self.fmt != 'lines' and self.fmt != 'sh':
             if self.section is None:
                 if self.conf.defaults():
                     self._print.section_header(iniparse.DEFAULTSECT)
@@ -972,9 +1151,16 @@ Options:
                 sections = (self.section,)
             if self.param is not None:
                 val = self.conf.get(self.section, self.param)
-                self._print.name_value(self.param, val, self.section)
+                print_section = self.section
+                if self.fmt == 'sh':
+                    print_section = None
+                self._print.name_value(self.param, val, print_section)
             else:
                 for section in sections:
+                    print_section = section
+                    if self.fmt == 'sh':
+                        if self.section or section == iniparse.DEFAULTSECT:
+                            print_section = None
                     if section == iniparse.DEFAULTSECT:
                         defaults_to_strip = {}
                     else:
@@ -985,23 +1171,24 @@ Options:
                         # if matching value also in default (global) section.
                         if defaults_to_strip.get(item[0]) != item[1]:
                             val = item[1]
-                            self._print.name_value(item[0], val, section)
+                            self._print.name_value(item[0], val, print_section)
                             items = True
-                    if not items:
-                        self._print.name_value(None, None, section)
+                    if not items and self.fmt != 'sh':
+                        self._print.name_value(None, None, print_section)
 
     def run(self):
         if not file_is_closed(sys.stdin) and sys.stdin.isatty():
             sys.excepthook = Crudini.cli_exception
 
         Crudini.init_iniparse_defaultsect()
-        self.parse_options()
+        operations = self.parse_options()
 
-        self.section_explicit_default = False
-        if self.section == '':
-            self.section = iniparse.DEFAULTSECT
-        elif self.section == iniparse.DEFAULTSECT:
-            self.section_explicit_default = True
+        # --set takes precedence to create file etc.
+        if self.mode == '--del':
+            for o in operations:
+                if o[0] == '--set':
+                    self.mode = '--set'
+                    break
 
         if self.mode == '--merge':
             self.mconf = self.parse_file('-', preserve_case=True)
@@ -1028,19 +1215,31 @@ Options:
             ):
                 self.added_default_section = self.madded_default_section
 
-            if self.mode == '--set':
-                self.command_set()
-            elif self.mode == '--merge':
-                self.command_merge()
-            elif self.mode == '--del':
-                self.command_del()
-            elif self.mode == '--get':
-                self.command_get()
+            for o in operations:
+                self.set_operation(o)
+
+                if self.mode == '--set':
+                    self.command_set()
+                elif self.mode == '--merge':
+                    self.command_merge()
+                elif self.mode == '--del':
+                    self.command_del()
+                elif self.mode == '--get':
+                    self.command_get()
 
             if self.mode != '--get':
+                # Del possible extraneous blank line left with removed section
+                # XXX: This may collapse existing multiple blank lines
+                if self.removed_section and 'sectionspace' not in self.iniopt:
+                    iniparse.tidy(self.conf)
+
                 # XXX: Ideally we should just do conf.write(f) here, but to
                 # avoid iniparse issues, we massage the data a little here
-                str_data = str(self.conf.data)
+                if sys.version_info[0] >= 3:
+                    str_data = str(self.conf.data)
+                else:
+                    # XXX: Can't use uc() here as can't specify encoding
+                    str_data = unicode(self.conf.data)
                 if len(str_data) and str_data[-1] != '\n':
                     str_data += '\n'
 
@@ -1068,17 +1267,39 @@ Options:
                     else:
                         str_data = str_data.replace(default_sect, '', 1)
 
+                # Handle creation of non special "default" section
+                if self.default_adjust:
+                    str_data = str_data.replace('crudini_default_adjust_', '')
+
+                # Process blank lines around sections
+                if 'sectionspace' in self.iniopt:
+                    # Ensure a single blank line before sections
+                    str_data = re.sub(r'\n*(\[[^\]]+\])', r'\n\n\1', str_data)
+                    str_data = str_data.lstrip('\n')  # remove leading \n
+                    str_data = str_data.rstrip('\n') + '\n'  # ensure \n at end
+                else:
+                    # Remove extraneous blanks iniparse adds when adding sects
+                    for section in self.ini_section_blanks:
+                        section_ = '\n[%s]\n' % section
+                        str_data = str_data.replace(section_, section_[1:], 1)
+
                 if self.windows_eol:
                     # iniparse uses '\n' for new/updated items
                     # so reset all to windows format
                     str_data = str_data.replace('\r\n', '\n')
                     str_data = str_data.replace('\n', '\r\n')
 
+                if self.indented:
+                    str_data = self.replace_leading.sub(r'\2\1', str_data)
+
                 if self.crudini_no_arg:
                     spacing = '' if 'nospace' in self.iniopt else ' '
                     str_data = str_data.replace('%s=%scrudini_no_arg' %
                                                 (spacing, spacing), '')
 
+                if self.bom:
+                    str_data = u'\ufeff%s' % str_data
+
                 changed = self.chksum != self._chksum(str_data)
 
                 if self.output == '-':
