@@ -54,25 +54,27 @@ crudini --set test.ini nonDEFAULT name val
 printf '%s\n' '[nonDEFAULT]' 'name = val' > good.ini
 diff -u test.ini good.ini && ok || fail
 
-printf '%s\n' 'global=val' > test.ini
-crudini --set test.ini '' global valnew
-printf '%s\n' 'global=valnew' > good.ini
-diff -u test.ini good.ini && ok || fail
+for bom in '' $'\xef\xbb\xbf'; do
+  printf '%s%s\n' "$bom" 'global=val' > test.ini
+  crudini --set test.ini '' global valnew
+  printf '%s%s\n' "$bom" 'global=valnew' > good.ini
+  diff -u test.ini good.ini && ok || fail
 
-printf '%s\n' 'global=val' > test.ini
-crudini --set test.ini DEFAULT global valnew
-printf '%s\n' '[DEFAULT]' 'global=valnew' > good.ini
-diff -u test.ini good.ini && ok || fail
+  printf '%s%s\n' "$bom" 'global=val' > test.ini
+  crudini --set test.ini DEFAULT global valnew
+  printf '%s%s\n' "$bom" '[DEFAULT]' 'global=valnew' > good.ini
+  diff -u test.ini good.ini && ok || fail
 
-printf '%s\n' '[DEFAULT]' 'global=val' > test.ini
-crudini --set test.ini DEFAULT global valnew
-printf '%s\n' '[DEFAULT]' 'global=valnew' > good.ini
-diff -u test.ini good.ini && ok || fail
+  printf '%s%s\n' "$bom" '[DEFAULT]' 'global=val' > test.ini
+  crudini --set test.ini DEFAULT global valnew
+  printf '%s%s\n' "$bom" '[DEFAULT]' 'global=valnew' > good.ini
+  diff -u test.ini good.ini && ok || fail
 
-printf '%s\n' 'global=val' '' '[nonDEFAULT]' 'name=val' > test.ini
-crudini --set test.ini '' global valnew
-printf '%s\n' 'global=valnew' '' '[nonDEFAULT]' 'name=val' > good.ini
-diff -u test.ini good.ini && ok || fail
+  printf '%s%s\n' "$bom" 'global=val' '' '[nonDEFAULT]' 'name=val' > test.ini
+  crudini --set test.ini '' global valnew
+  printf '%s%s\n' "$bom" 'global=valnew' '' '[nonDEFAULT]' 'name=val' > good.ini
+  diff -u test.ini good.ini && ok || fail
+done
 
 # do these --sets which test [DEFAULT] handling also with --inplace
 for mode in '' '--inplace'; do
@@ -186,6 +188,9 @@ diff -u test.ini good.ini && ok || fail
 # basic get
 test "$(crudini --get example.ini section1 cAps)" = 'not significant' && ok || fail
 
+# unicode get
+test "$(crudini --get example.ini non-sh-compat útf8name)" = 'val' && ok || fail
+
 # get sections
 crudini --get example.ini > test.ini
 printf '%s\n' DEFAULT section1 'empty section' non-sh-compat list > good.ini
@@ -541,6 +546,10 @@ test "$(crudini --output=- --set noequals.ini noequals param1 '' \
 test "$(crudini --ini-options=nospace --output=- --set noequals.ini \
         noequals param1 value1 \
         | grep param1)" = 'param1=value1' && ok || fail
+#  Ensure correct spacing with --ini-options=space
+test "$(crudini --ini-options=space --output=- --set noequals.ini \
+        noequals param1 value1 \
+        | grep param1)" = 'param1 = value1' && ok || fail
 
 # Test can read windows format files
 printf '%s\r\n' '' 'empty' 'param=value' > test.ini
@@ -570,6 +579,13 @@ diff -u <(crudini --output=- --ini-options=nospace \
 diff -u <(crudini --output=- --ini-options=nospace \
           --set nospace-in.ini '' new val) \
         nospace-out.ini && ok || fail
+# Test --ini-options=space
+diff -u <(crudini --output=- --ini-options=space \
+          --set space-out.ini '' new val) \
+        space-out.ini && ok || fail
+# - Mixed space / nospace not allowed
+crudini --get file.conf '' param1 --ini-options=space,nospace \
+        2>/dev/null && fail || ok
 
 # Test multi operation
 # - Multiple set
@@ -674,3 +690,29 @@ crudini --ini-options=ignoreindent \
         --set file.conf '' '  param1' a && ok || fail
 diff -u good.conf file.conf && ok || fail
 rm file.conf good.conf
+
+# Test removal of extraneous empty lines
+# implicitly enabled with --del
+printf '\n[%s]\n' 1 2 > good.conf
+cp good.conf file.conf
+for i in 1 2; do crudini --del file.conf $i; crudini --set file.conf $i; done
+diff -u good.conf file.conf && ok || fail
+rm file.conf good.conf
+
+# Test addition/removal of empty lines with sectionspace
+printf '\n[%s]\n[%s]\n\n[%s]\n\n\n[%s]\n\n' 1 2 3 4 > file.conf
+printf '[%s]\n\n[%s]\n\n[%s]\n\n[%s]\n' 1 2 3 4 > good.conf
+crudini --set --ini-options=sectionspace file.conf ''
+diff -u good.conf file.conf && ok || fail
+rm file.conf good.conf
+
+# Test creation of a "default" section
+printf '[%s]\n' 'default' > good.conf
+crudini --set file.conf default # new
+diff -u good.conf file.conf && ok || fail
+crudini --set file.conf default # existing
+diff -u good.conf file.conf && ok || fail
+rm file.conf
+crudini --set file.conf default --set file.conf default # double new
+diff -u good.conf file.conf && ok || fail
+rm file.conf good.conf
